@@ -1,7 +1,8 @@
 /*****************************************************************************
- * libvlcjni.c
+ * eventfd.c
  *****************************************************************************
  * Copyright © 2012 Rafaël Carré
+ * Copyright © 2012 VLC authors and VideoLAN
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,28 +20,10 @@
  *****************************************************************************/
 
 #include <sys/linux-syscalls.h>
+#include <sys/syscall.h>
 #include <errno.h>
 
 int eventfd(unsigned int initval, int flags)
 {
-    int ret;
-    int syscall_nr = __NR_eventfd2;
-
-    asm(
-    "mov    r0, %[initval]      \n\t"
-    "mov    r1, %[flags]        \n\t"
-    "mov    r7, %[nr]           \n\t"
-    "svc    #0                  \n\t"
-    "mov    %[ret], r0          \n\t"
-    : [ret] "=r" (ret)
-    : [initval] "r" (initval), [flags] "r" (flags), [nr] "r" (syscall_nr)
-    : "r7"
-    );
-
-    if (ret < 0) {
-        errno = -ret;
-        return -1;
-    }
-
-    return ret;
+    return syscall(__NR_eventfd2, initval, flags);
 }
