@@ -162,7 +162,7 @@ libvlc_media_t *new_media(jlong instance, JNIEnv *env, jobject thiz, jstring fil
              */
             libvlc_media_add_option(p_md, ":file-caching=1500");
             libvlc_media_add_option(p_md, ":network-caching=1500");
-            libvlc_media_add_option(p_md, ":codec=iomx,all");
+            libvlc_media_add_option(p_md, ":codec=mediacodec,iomx,all");
         }
         if (noVideo)
             libvlc_media_add_option(p_md, ":no-video");
@@ -424,6 +424,7 @@ void Java_org_videolan_vlc_LibVLC_nativeInit(JNIEnv *env, jobject thiz, jboolean
         "--no-plugins-cache",
         "--no-drop-late-frames",
         "--avcodec-fast",
+        "--avcodec-threads=0",
         enable_time_stretch ? "--audio-time-stretch" : "--no-audio-time-stretch",
         use_opensles ? "--aout=opensles" : "--aout=android_audiotrack",
     };
@@ -617,7 +618,7 @@ jint Java_org_videolan_vlc_LibVLC_readMedia(JNIEnv *env, jobject thiz,
     if (!m)
     {
         LOGE("readMedia: Could not create the media!");
-        return;
+        return -1;
     }
 
     libvlc_media_list_t* p_mlist = getMediaList(env, thiz);
@@ -627,7 +628,7 @@ jint Java_org_videolan_vlc_LibVLC_readMedia(JNIEnv *env, jobject thiz,
         LOGE("readMedia: Could not add to the media list!");
         libvlc_media_list_unlock(p_mlist);
         libvlc_media_release(m);
-        return;
+        return -1;
     }
     int position = libvlc_media_list_index_of_item(p_mlist, m);
     libvlc_media_list_unlock(p_mlist);
