@@ -20,9 +20,6 @@
 
 package org.videolan.vlc;
 
-import java.io.File;
-import java.math.BigInteger;
-import java.security.MessageDigest;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
@@ -30,36 +27,31 @@ import java.util.Stack;
 
 import org.videolan.vlc.gui.MainActivity;
 import org.videolan.vlc.gui.audio.AudioPlayerActivity;
+import org.videolan.vlc.gui.audio.AudioUtil;
 import org.videolan.vlc.gui.video.VideoPlayerActivity;
 import org.videolan.vlc.interfaces.IAudioService;
 import org.videolan.vlc.interfaces.IAudioServiceCallback;
 
-import android.annotation.SuppressLint;
 import android.annotation.TargetApi;
-import android.app.Notification;
 import android.app.PendingIntent;
 import android.app.Service;
 import android.content.BroadcastReceiver;
 import android.content.ComponentName;
-import android.content.ContentResolver;
 import android.content.Context;
 import android.content.Intent;
 import android.content.IntentFilter;
-import android.database.Cursor;
 import android.graphics.Bitmap;
-import android.graphics.BitmapFactory;
 import android.media.AudioManager;
 import android.media.AudioManager.OnAudioFocusChangeListener;
 import android.media.MediaMetadataRetriever;
 import android.media.RemoteControlClient;
 import android.media.RemoteControlClient.MetadataEditor;
-import android.net.Uri;
 import android.os.Handler;
 import android.os.IBinder;
 import android.os.Message;
 import android.os.RemoteException;
 import android.os.SystemClock;
-import android.provider.MediaStore;
+import android.support.v4.app.NotificationCompat;
 import android.telephony.TelephonyManager;
 import android.util.Log;
 import android.view.KeyEvent;
@@ -85,12 +77,12 @@ public class AudioService extends Service {
     private Media mCurrentMedia;
     private HashMap<IAudioServiceCallback, Integer> mCallback;
     private EventManager mEventManager;
-    private Notification mNotification;
     private boolean mShuffling = false;
     private RepeatType mRepeating = RepeatType.None;
     private boolean mDetectHeadset = true;
     private long mHeadsetDownTime = 0;
     private long mHeadsetUpTime = 0;
+    private OnAudioFocusChangeListener audioFocusListener;
 
     /**
      * RemoteControlClient is for lock screen playback control.
@@ -137,6 +129,8 @@ public class AudioService extends Service {
 
         if(Util.isICSOrLater())
             setUpRemoteControlClient();
+
+        AudioUtil.prepareCacheFolder();
     }
 
     @TargetApi(14)
@@ -186,8 +180,8 @@ public class AudioService extends Service {
 
     @Override
     public void onDestroy() {
-        unregisterReceiver(serviceReceiver);
         super.onDestroy();
+        unregisterReceiver(serviceReceiver);
     }
 
     @Override
@@ -200,35 +194,36 @@ public class AudioService extends Service {
         if(!Util.isGingerbreadOrLater()) // NOP if not supported
             return;
 
+        audioFocusListener = new OnAudioFocusChangeListener() {
+            int volume = -1;
+            @Override
+            public void onAudioFocusChange(int focusChange) {
+                AudioManager am = (AudioManager)getSystemService(AUDIO_SERVICE);
+                if(focusChange == AudioManager.AUDIOFOCUS_LOSS_TRANSIENT_CAN_DUCK ||
+                   focusChange == AudioManager.AUDIOFOCUS_LOSS_TRANSIENT) {
+                    volume = am.getStreamVolume(AudioManager.STREAM_MUSIC);
+                    int maxVol = am.getStreamMaxVolume(AudioManager.STREAM_MUSIC);
+                    /*
+                     * Lower the volume to 19% to "duck" when an alert or something
+                     * needs to be played.
+                     */
+                    am.setStreamVolume(AudioManager.STREAM_MUSIC, (int)(0.19*maxVol), 0);
+                } else {
+                    if(volume != -1) {
+                        am.setStreamVolume(AudioManager.STREAM_MUSIC, volume, 0);
+                        volume = -1;
+                    }
+                }
+            }
+        };
+
         AudioManager am = (AudioManager)getSystemService(AUDIO_SERVICE);
         if(gain)
             am.requestAudioFocus(audioFocusListener, AudioManager.STREAM_MUSIC, AudioManager.AUDIOFOCUS_GAIN);
         else
             am.abandonAudioFocus(audioFocusListener);
-    }
 
-    private final OnAudioFocusChangeListener audioFocusListener = new OnAudioFocusChangeListener() {
-        int volume = -1;
-        @Override
-        public void onAudioFocusChange(int focusChange) {
-            AudioManager am = (AudioManager)getSystemService(AUDIO_SERVICE);
-            if(focusChange == AudioManager.AUDIOFOCUS_LOSS_TRANSIENT_CAN_DUCK ||
-               focusChange == AudioManager.AUDIOFOCUS_LOSS_TRANSIENT) {
-                volume = am.getStreamVolume(AudioManager.STREAM_MUSIC);
-                int maxVol = am.getStreamMaxVolume(AudioManager.STREAM_MUSIC);
-                /*
-                 * Lower the volume to 19% to "duck" when an alert or something
-                 * needs to be played.
-                 */
-                am.setStreamVolume(AudioManager.STREAM_MUSIC, (int)(0.19*maxVol), 0);
-            } else {
-                if(volume != -1) {
-                    am.setStreamVolume(AudioManager.STREAM_MUSIC, volume, 0);
-                    volume = -1;
-                }
-            }
-        }
-    };
+    }
 
     private final BroadcastReceiver serviceReceiver = new BroadcastReceiver() {
         @Override
@@ -264,13 +259,17 @@ public class AudioService extends Service {
                 next();
             }
 
+            // skip all headsets events if there is a call
+            TelephonyManager telManager = (TelephonyManager) getSystemService(Context.TELEPHONY_SERVICE);
+            if (telManager != null && telManager.getCallState() != TelephonyManager.CALL_STATE_IDLE)
+                return;
+
             /*
              * headset controller events
              */
             else if (action.equalsIgnoreCase(Intent.ACTION_MEDIA_BUTTON)) {
                 KeyEvent event = (KeyEvent) intent.getParcelableExtra(Intent.EXTRA_KEY_EVENT);
-                TelephonyManager telManager = (TelephonyManager) getSystemService(Context.TELEPHONY_SERVICE);
-                if (event == null || telManager.getCallState() != TelephonyManager.CALL_STATE_IDLE)
+                if (event == null)
                     return;
 
                 if (mCurrentMedia == null) {
@@ -456,23 +455,27 @@ public class AudioService extends Service {
 
     private void showNotification() {
         // add notification to status bar
-        if (mNotification == null) {
-            mNotification = new Notification(R.drawable.icon, null,
-                    System.currentTimeMillis());
-        }
+        NotificationCompat.Builder builder = new NotificationCompat.Builder(this)
+            .setSmallIcon(R.drawable.icon)
+            .setLargeIcon(AudioUtil.getCover(this, mCurrentMedia, 64))
+            .setContentTitle(mCurrentMedia.getTitle())
+            .setContentText((Util.isJellyBeanOrLater() ? mCurrentMedia.getArtist()
+                    : mCurrentMedia.getArtist() + " - " + mCurrentMedia.getAlbum()))
+            .setContentInfo(mCurrentMedia.getAlbum())
+            .setAutoCancel(false)
+            .setOngoing(true);
+
         Intent notificationIntent = new Intent(this, AudioPlayerActivity.class);
         notificationIntent.setAction(Intent.ACTION_MAIN);
         notificationIntent.addCategory(Intent.CATEGORY_LAUNCHER);
         notificationIntent.putExtra(START_FROM_NOTIFICATION, true);
         PendingIntent pendingIntent = PendingIntent.getActivity(this, 0, notificationIntent, 0);
-        mNotification.setLatestEventInfo(this, mCurrentMedia.getTitle(),
-                mCurrentMedia.getArtist() + " - " + mCurrentMedia.getAlbum(), pendingIntent);
-        startForeground(3, mNotification);
 
+        builder.setContentIntent(pendingIntent);
+        startForeground(3, builder.build());
     }
 
     private void hideNotification() {
-        mNotification = null;
         stopForeground(true);
     }
 
@@ -588,78 +591,8 @@ public class AudioService extends Service {
         mRepeating = RepeatType.values()[t];
     }
 
-    @SuppressLint("SdCardPath")
     private Bitmap getCover() {
-        try {
-            // try to get the cover from android MediaStore
-            ContentResolver contentResolver = getContentResolver();
-            Uri uri = android.provider.MediaStore.Audio.Albums.EXTERNAL_CONTENT_URI;
-            Cursor cursor = contentResolver.query(uri, new String[] {
-                           MediaStore.Audio.Albums.ALBUM,
-                           MediaStore.Audio.Albums.ALBUM_ART },
-                           MediaStore.Audio.Albums.ALBUM + " LIKE ?",
-                           new String[] { mCurrentMedia.getAlbum() }, null);
-            if (cursor == null) {
-                // do nothing
-            } else if (!cursor.moveToFirst()) {
-                // do nothing
-                cursor.close();
-            } else {
-                int titleColumn = cursor.getColumnIndex(android.provider.MediaStore.Audio.Albums.ALBUM_ART);
-                String albumArt = cursor.getString(titleColumn);
-                cursor.close();
-                if(albumArt != null) { // could be null (no album art stored)
-                    Bitmap b = BitmapFactory.decodeFile(albumArt);
-                    if (b != null)
-                        return b;
-                }
-            }
-
-            //cover not in MediaStore, trying vlc
-            String artworkURL = mCurrentMedia.getArtworkURL();
-            final String cacheDir = "/sdcard/Android/data/org.videolan.vlc/cache";
-            if (artworkURL != null && artworkURL.startsWith("file://")) {
-                return BitmapFactory.decodeFile(Uri.decode(artworkURL).replace("file://", ""));
-            } else if(artworkURL != null && artworkURL.startsWith("attachment://")) {
-                // Decode if the album art is embedded in the file
-                String mArtist = mCurrentMedia.getArtist();
-                String mAlbum = mCurrentMedia.getAlbum();
-
-                /* Parse decoded attachment */
-                if( mArtist.length() == 0 || mAlbum.length() == 0 ||
-                    mArtist.equals(VLCApplication.getAppContext().getString(R.string.unknown_artist)) ||
-                    mAlbum.equals(VLCApplication.getAppContext().getString(R.string.unknown_album)) )
-                {
-                    /* If artist or album are missing, it was cached by title MD5 hash */
-                    MessageDigest md = MessageDigest.getInstance("MD5");
-                    byte[] binHash = md.digest((artworkURL + mCurrentMedia.getTitle()).getBytes("UTF-8"));
-                    /* Convert binary hash to normal hash */
-                    BigInteger hash = new BigInteger(1, binHash);
-                    String titleHash = hash.toString(16);
-                    while(titleHash.length() < 32) {
-                        titleHash = "0" + titleHash;
-                    }
-                    /* Use generated hash to find art */
-                    artworkURL = cacheDir + "/art/arturl/" + titleHash + "/art.png";
-                } else {
-                    /* Otherwise, it was cached by artist and album */
-                    artworkURL = cacheDir + "/art/artistalbum/" + mArtist + "/" + mAlbum + "/art.png";
-                }
-
-                Log.v(TAG, "artworkURL (calculated) = " + artworkURL);
-                return BitmapFactory.decodeFile(artworkURL);
-            }
-
-            //still no cover found, looking in folder ...
-            File f = Util.URItoFile(mCurrentMedia.getLocation());
-            for (File s : f.getParentFile().listFiles()) {
-                if (s.getAbsolutePath().endsWith("png") ||
-                        s.getAbsolutePath().endsWith("jpg"))
-                    return BitmapFactory.decodeFile(s.getAbsolutePath());
-            }
-        } catch (Exception e) {
-        }
-        return null;
+        return AudioUtil.getCover(this, mCurrentMedia, 0);
     }
 
     private final IAudioService.Stub mInterface = new IAudioService.Stub() {
