@@ -32,12 +32,13 @@ import java.util.Stack;
 
 import org.videolan.vlc.gui.MainActivity;
 import org.videolan.vlc.gui.audio.AudioBrowserFragment;
-import org.videolan.vlc.gui.video.VideoListFragment;
+import org.videolan.vlc.gui.video.VideoGridFragment;
 
 import android.content.Context;
 import android.content.SharedPreferences;
 import android.os.Environment;
 import android.os.Handler;
+import android.os.Message;
 import android.preference.PreferenceManager;
 import android.util.Log;
 
@@ -47,23 +48,34 @@ public class MediaLibrary {
     public static final int MEDIA_ITEMS_UPDATED = 100;
 
     private static MediaLibrary mInstance;
-    private final DatabaseManager mDBManager;
     private final ArrayList<Media> mItemList;
     private final ArrayList<Handler> mUpdateHandler;
     private boolean isStopping = false;
+    private boolean mRestart = false;
+    private Context mRestartContext;
     protected Thread mLoadingThread;
 
     private MediaLibrary(Context context) {
         mInstance = this;
         mItemList = new ArrayList<Media>();
         mUpdateHandler = new ArrayList<Handler>();
-        mDBManager = DatabaseManager.getInstance(context);
+    }
+
+    public void loadMediaItems(Context context, boolean restart) {
+        if (restart && isWorking()) {
+            /* do a clean restart if a scan is ongoing */
+            mRestart = true;
+            isStopping = true;
+            mRestartContext = context;
+        } else {
+            loadMediaItems(context);
+        }
     }
 
     public void loadMediaItems(Context context) {
         if (mLoadingThread == null || mLoadingThread.getState() == State.TERMINATED) {
             isStopping = false;
-            VideoListFragment.actionScanStart(context.getApplicationContext());
+            VideoGridFragment.actionScanStart(context.getApplicationContext());
             mLoadingThread = new Thread(new GetMediaItemsRunnable(context.getApplicationContext()));
             mLoadingThread.start();
         }
@@ -73,6 +85,14 @@ public class MediaLibrary {
         isStopping = true;
     }
 
+    public boolean isWorking() {
+        if (mLoadingThread != null &&
+            mLoadingThread.getState() != State.TERMINATED &&
+            mLoadingThread.getState() != State.NEW)
+            return true;
+        return false;
+    }
+
     public static MediaLibrary getInstance(Context context) {
         if (mInstance == null)
             mInstance = new MediaLibrary(context);
@@ -160,10 +180,6 @@ public class MediaLibrary {
         return items;
     }
 
-    public boolean ismLoadingThreadrunning () {
-        return mLoadingThread.isAlive();
-    };
-
     private class GetMediaItemsRunnable implements Runnable {
 
         private final Stack<File> directories = new Stack<File>();
@@ -177,7 +193,8 @@ public class MediaLibrary {
         @Override
         public void run() {
             // Initialize variables
-            SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(mContext);
+            final DatabaseManager DBManager = DatabaseManager.getInstance(VLCApplication.getAppContext());
+            final SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(mContext);
 
             String root = pref.getString("directories_root", null);
 
@@ -192,13 +209,13 @@ public class MediaLibrary {
             MainActivity.showProgressBar(mContext);
 
             // get directories from database
-            mDBManager.removeDirNotUnder(root);
-            directories.addAll(mDBManager.getMediaDirs());
+            DBManager.removeDirNotUnder(root);
+            directories.addAll(DBManager.getMediaDirs());
             if (directories.isEmpty())
                 directories.add(new File(root));
 
             // get all existing media items
-            HashMap<String, Media> existingMedias = mDBManager.getMedias(mContext);
+            HashMap<String, Media> existingMedias = DBManager.getMedias(mContext);
 
             // list of all added files
             HashSet<String> addedLocations = new HashSet<String>();
@@ -239,7 +256,7 @@ public class MediaLibrary {
                     }
 
                     // Filter the extensions and the folders
-                    try { 
+                    try {
                         if ((f = dir.listFiles(mediaFileFilter)) != null) {
                             for (File file : f) {
                                 if (file.isFile()) {
@@ -298,24 +315,46 @@ public class MediaLibrary {
                     for (String fileURI : addedLocations) {
                         existingMedias.remove(fileURI);
                     }
-                    mDBManager.removeMedias(existingMedias.keySet());
+                    DBManager.removeMedias(existingMedias.keySet());
 
-                    for (File file : mDBManager.getMediaDirs())
+                    for (File file : DBManager.getMediaDirs())
                         if (!file.isDirectory())
-                            mDBManager.removeDir(file.getAbsolutePath());
+                            DBManager.removeDir(file.getAbsolutePath());
                 }
 
                 // hide progressbar in footer
                 MainActivity.clearTextInfo(mContext);
                 MainActivity.hideProgressBar(mContext);
 
-                VideoListFragment.actionScanStop(mContext);
+                VideoGridFragment.actionScanStop(mContext);
 
-                mContext = null;
+                if (mRestart) {
+                    Log.d(TAG, "Restarting scan");
+                    mRestart = false;
+                    restartHandler.sendEmptyMessage(1);
+                } else {
+                    mRestartContext = null;
+                    mContext = null;
+                }
             }
         }
     };
 
+    private Handler restartHandler = new Handler() {
+        @Override
+        public void handleMessage(final Message msgs) {
+            restartHandler.postDelayed(new Runnable() {
+                @Override
+                public void run() {
+                    if (mRestartContext != null)
+                        loadMediaItems(mRestartContext);
+                    else
+                        Log.e(TAG, "Context lost in a black hole");
+                }
+            }, 200);
+        }
+    };
+
     /**
      * Filters all irrelevant files
      */
