@@ -72,10 +72,7 @@ public class Util {
     }
 
     public static String URItoFileName(String URI) {
-        int sep = URI.lastIndexOf('/');
-        int dot = URI.lastIndexOf('.');
-        String name = dot >= 0 ? URI.substring(sep + 1, dot) : URI;
-        return Uri.decode(name);
+        return URItoFile(URI).getName();
     }
 
     public static String PathToURI(String path) {
@@ -88,10 +85,9 @@ public class Util {
         return URI;
     }
 
-    public static String stripTrailingSlash(String _s) {
-        String s = _s;
+    public static String stripTrailingSlash(String s) {
         if( s.endsWith("/") && s.length() > 1 )
-            s = s.substring(0,s.length()-1);
+            return s.substring(0, s.length() - 1);
         return s;
     }
 
@@ -232,6 +228,11 @@ public class Util {
         return android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.ICE_CREAM_SANDWICH;
     }
 
+    public static boolean isJellyBeanOrLater()
+    {
+        return android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.JELLY_BEAN;
+    }
+
     public static boolean hasNavBar()
     {
         return hasNavBar;
@@ -267,7 +268,6 @@ public class Util {
         }
 
         String ANDROID_ABI = properties.getProperty("ANDROID_ABI");
-        boolean NO_NEON = properties.getProperty("NO_NEON","0").equals("1");
         boolean NO_FPU = properties.getProperty("NO_FPU","0").equals("1");
         boolean NO_ARMV6 = properties.getProperty("NO_ARMV6","0").equals("1");
         boolean hasNeon = false, hasFpu = false, hasArmV6 = false, hasArmV7 = false;
@@ -341,10 +341,8 @@ public class Util {
             }
         }
         if(ANDROID_ABI.equals("armeabi") || ANDROID_ABI.equals("armeabi-v7a")) {
-            if(!NO_NEON && !hasNeon) {
+            if(!hasNeon) {
                 errorMsg = "NEON build on non-NEON device";
-                isCompatible = false;
-                return false;
             }
         }
         errorMsg = null;
