@@ -53,6 +53,9 @@ public class DirectoryViewFragment extends SherlockListFragment implements ISort
 
     private DirectoryAdapter mDirectoryAdapter;
 
+    /* All subclasses of Fragment must include a public empty constructor. */
+    public DirectoryViewFragment() { }
+
     @Override
     public void onCreate(Bundle savedInstanceState) {
         super.onCreate(savedInstanceState);
@@ -141,6 +144,17 @@ public class DirectoryViewFragment extends SherlockListFragment implements ISort
         }
     }
 
+    public boolean isRootDirectory () {
+        if (mDirectoryAdapter.getmCurrentDir().equals(mDirectoryAdapter.getmRootDir())) {
+            return true;
+        }
+        return false;
+    };
+
+    public void showParentDirectory() {
+            mDirectoryAdapter.browse("..");
+    };
+
     private void openMediaFile(int p) {
         AudioServiceController audioController = AudioServiceController.getInstance();
         String mediaFile = mDirectoryAdapter.getMediaLocation(p);
