@@ -28,10 +28,12 @@ import org.videolan.vlc.LibVLC;
 import org.videolan.vlc.LibVlcException;
 import org.videolan.vlc.MediaLibrary;
 import org.videolan.vlc.R;
+import org.videolan.vlc.ThumbnailerManager;
 import org.videolan.vlc.Util;
 import org.videolan.vlc.VLCCallbackTask;
 import org.videolan.vlc.gui.SidebarAdapter.SidebarEntry;
 import org.videolan.vlc.gui.video.VideoListAdapter;
+import org.videolan.vlc.gui.video.VideoGridFragment;
 import org.videolan.vlc.interfaces.ISortable;
 import org.videolan.vlc.widget.AudioMiniPlayer;
 
@@ -86,11 +88,15 @@ public class MainActivity extends SherlockFragmentActivity {
     private static final String PREF_SHOW_INFO = "show_info";
     private static final String PREF_FIRST_RUN = "first_run";
 
+    private static final int ACTIVITY_RESULT_PREFERENCES = 1;
+
     private ActionBar mActionBar;
     private SlidingMenu mMenu;
     private SidebarAdapter mSidebarAdapter;
     private AudioMiniPlayer mAudioPlayer;
     private AudioServiceController mAudioController;
+    private ThumbnailerManager mThumbnailerManager;
+
     private View mInfoLayout;
     private ProgressBar mInfoProgress;
     private TextView mInfoText;
@@ -100,35 +106,35 @@ public class MainActivity extends SherlockFragmentActivity {
 
     private int mVersionNumber = -1;
     private boolean mFirstRun = false;
-
-    public MainActivity() {
-    }
+    private boolean mScanNeeded = true;
 
     @Override
     protected void onCreate(Bundle savedInstanceState) {
+        super.onCreate(savedInstanceState);
+
         if (!Util.hasCompatibleCPU()) {
             Log.e(TAG, Util.getErrorMsg());
-            super.onCreate(savedInstanceState);
             Intent i = new Intent(this, CompatErrorActivity.class);
             startActivity(i);
             finish();
             return;
         }
 
-        if (Util.isICSOrLater()) /* Bug on pre-ICS, the progress bar is always present */
-            requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);
+        /* Enable the indeterminate progress feature */
+        requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);
 
         // Set up the sliding menu
         setContentView(R.layout.sliding_menu);
         mMenu = (SlidingMenu) findViewById(R.id.sliding_menu);
-        updateMenuOffset();
+        changeMenuOffset();
 
         View v_main = LayoutInflater.from(this).inflate(R.layout.main, null);
         mMenu.setContent(v_main);
+
         View sidebar = LayoutInflater.from(this).inflate(R.layout.sidebar, null);
-        ((ListView)sidebar).setFooterDividersEnabled(true);
         final ListView listView = (ListView)sidebar.findViewById(android.R.id.list);
-        mSidebarAdapter = new SidebarAdapter(getSupportFragmentManager());
+        listView.setFooterDividersEnabled(true);
+        mSidebarAdapter = new SidebarAdapter();
         listView.setAdapter(mSidebarAdapter);
         mMenu.setMenu(sidebar);
 
@@ -153,13 +159,14 @@ public class MainActivity extends SherlockFragmentActivity {
             editor.commit();
         }
 
+        /* Start VLC prefs */
         LibVLC.useIOMX(this);
+
         try {
             // Start LibVLC
             LibVLC.getInstance();
         } catch (LibVlcException e) {
             e.printStackTrace();
-            super.onCreate(null);
             Intent i = new Intent(this, CompatErrorActivity.class);
             i.putExtra("runtimeError", true);
             i.putExtra("message", "LibVLC failed to initialize (LibVlcException)");
@@ -168,9 +175,7 @@ public class MainActivity extends SherlockFragmentActivity {
             return;
         }
 
-        super.onCreate(savedInstanceState);
-
-        /* Initialize variables */
+        /* Initialize UI variables */
         mInfoLayout = v_main.findViewById(R.id.info_layout);
         mInfoProgress = (ProgressBar) v_main.findViewById(R.id.info_progress);
         mInfoText = (TextView) v_main.findViewById(R.id.info_text);
@@ -191,7 +196,7 @@ public class MainActivity extends SherlockFragmentActivity {
         listView.setOnItemClickListener(new OnItemClickListener() {
 
             @Override
-            public void onItemClick (AdapterView<?> parent, View view,
+            public void onItemClick(AdapterView<?> parent, View view,
                     int position, long id) {
                 SidebarAdapter.SidebarEntry entry = (SidebarEntry) listView.getItemAtPosition(position);
                 Fragment current = getSupportFragmentManager().findFragmentById(R.id.fragment_placeholder);
@@ -201,19 +206,18 @@ public class MainActivity extends SherlockFragmentActivity {
                     return;
                 }
 
-                /* Clear any backstack before switching tabs.
-                 * This way it's more consistent for the user, who might have
-                 * switched tabs and hit back to quit, only to activate an old
-                 * backstack.
+                /*
+                 * Clear any backstack before switching tabs. This avoids
+                 * activating an old backstack, when a user hits the back button
+                 * to quit
                  */
-                if(getSupportFragmentManager().getBackStackEntryCount() > 0) {
-                    for(int i = 0; i < getSupportFragmentManager().getBackStackEntryCount(); i++) {
-                        getSupportFragmentManager().popBackStack();
-                    }
+                for(int i = 0; i < getSupportFragmentManager().getBackStackEntryCount(); i++) {
+                    getSupportFragmentManager().popBackStack();
                 }
+
                 FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
                 ft.detach(current);
-                ft.attach(mSidebarAdapter.getFragment(entry.id));
+                ft.attach(getFragment(entry.id));
                 ft.commit();
                 mCurrentFragment = entry.id;
                 mMenu.showAbove();
@@ -258,11 +262,12 @@ public class MainActivity extends SherlockFragmentActivity {
         /* Reload the latest preferences */
         reloadPreferences();
 
-        /* Load media items from database and storage */
-        MediaLibrary.getInstance(this).loadMediaItems(this);
+        /* Load the thumbnailer */
+        mThumbnailerManager = new ThumbnailerManager(this,
+                getWindowManager().getDefaultDisplay());
     }
 
-    private void updateMenuOffset() {
+    private void changeMenuOffset() {
         WindowManager wm = (WindowManager) getSystemService(WINDOW_SERVICE);
         Display display = wm.getDefaultDisplay();
         @SuppressWarnings("deprecation")
@@ -272,9 +277,24 @@ public class MainActivity extends SherlockFragmentActivity {
 
     @Override
     protected void onResume() {
+        super.onResume();
         mAudioController.addAudioPlayer(mAudioPlayer);
         AudioServiceController.getInstance().bindAudioService(this);
-        Boolean startFromNotification = getIntent().hasExtra(AudioService.START_FROM_NOTIFICATION);
+
+        /* FIXME: this is used to avoid having MainActivity twice in the backstack */
+        if (getIntent().hasExtra(AudioService.START_FROM_NOTIFICATION))
+            getIntent().removeExtra(AudioService.START_FROM_NOTIFICATION);
+
+        /* Load media items from database and storage */
+        if (mScanNeeded)
+            MediaLibrary.getInstance(this).loadMediaItems(this);
+    }
+
+
+
+    @Override
+    protected void onResumeFragments() {
+        super.onResumeFragments();
 
         /* Restore last view */
         Fragment current = getSupportFragmentManager()
@@ -282,7 +302,7 @@ public class MainActivity extends SherlockFragmentActivity {
         boolean found = false;
         if(current != null) {
             for(int i = 0; i < SidebarAdapter.entries.size(); i++) {
-                if(SidebarAdapter.entries.get(i).id == current.getTag()) {
+                if(SidebarAdapter.entries.get(i).id.equals(current.getTag())) {
                     found = true;
                     break;
                 }
@@ -296,13 +316,9 @@ public class MainActivity extends SherlockFragmentActivity {
          */
         if(found) {
             FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
-            ft.replace(R.id.fragment_placeholder, mSidebarAdapter.getFragment(mCurrentFragment));
+            ft.replace(R.id.fragment_placeholder, getFragment(mCurrentFragment));
             ft.commit();
         }
-
-        if (startFromNotification)
-            getIntent().removeExtra(AudioService.START_FROM_NOTIFICATION);
-        super.onResume();
     }
 
     /**
@@ -310,22 +326,31 @@ public class MainActivity extends SherlockFragmentActivity {
      */
     @Override
     protected void onPause() {
+        super.onPause();
+
+        /* Check for an ongoing scan that needs to be resumed during onResume */
+        mScanNeeded = MediaLibrary.getInstance(this).isWorking();
         /* Stop scanning for files */
         MediaLibrary.getInstance(this).stop();
-
+        /* Stop the thumbnailer */
+        mThumbnailerManager.stop();
+        /* Save the tab status in pref */
         SharedPreferences.Editor editor = getSharedPreferences("MainActivity", MODE_PRIVATE).edit();
         editor.putString("fragment", mCurrentFragment);
         editor.commit();
+
         mAudioController.removeAudioPlayer(mAudioPlayer);
-        super.onPause();
     }
 
     @Override
     protected void onDestroy() {
+        super.onDestroy();
+
         try {
             unregisterReceiver(messageReceiver);
         } catch (IllegalArgumentException e) {}
-        super.onDestroy();
+        if (mThumbnailerManager != null)
+            mThumbnailerManager.clearJobs();
     }
 
     @Override
@@ -340,9 +365,34 @@ public class MainActivity extends SherlockFragmentActivity {
         if(mMenu.isBehindShowing()) {
             /* Close the menu first */
             mMenu.showAbove();
-        } else {
-            super.onBackPressed();
+            return;
         }
+        // If it's the directory view, a "backpressed" action shows a parent.
+        if (mCurrentFragment.equals("directories")) {
+            DirectoryViewFragment directoryView = (DirectoryViewFragment) getFragment(mCurrentFragment);
+            if (!directoryView.isRootDirectory()) {
+                directoryView.showParentDirectory();
+                return;
+            }
+        }
+        super.onBackPressed();
+    }
+
+    private Fragment getFragment(String id)
+    {
+        Fragment fragment = mSidebarAdapter.fetchFragment(id);
+
+        if (!fragment.isAdded())
+            getSupportFragmentManager()
+                .beginTransaction()
+                .add(R.id.fragment_placeholder, fragment, id)
+                .commitAllowingStateLoss();
+
+        /* Start the thumbnailer */
+        if (id.equals("video"))
+            mThumbnailerManager.start((VideoGridFragment)fragment);
+
+        return fragment;
     }
 
     /** Create menu from XML
@@ -361,7 +411,7 @@ public class MainActivity extends SherlockFragmentActivity {
     @Override
     public void onConfigurationChanged(Configuration newConfig) {
         super.onConfigurationChanged(newConfig);
-        updateMenuOffset();
+        changeMenuOffset();
     }
 
     @Override
@@ -377,7 +427,7 @@ public class MainActivity extends SherlockFragmentActivity {
     @Override
     public boolean onOptionsItemSelected(MenuItem item) {
 
-        // Intent to start new Activity
+        // Intent to start a new Activity
         Intent intent;
 
         // Handle item selection
@@ -400,19 +450,19 @@ public class MainActivity extends SherlockFragmentActivity {
             // Preferences
             case R.id.ml_menu_preferences:
                 intent = new Intent(this, PreferencesActivity.class);
-                startActivity(intent);
+                startActivityForResult(intent, ACTIVITY_RESULT_PREFERENCES);
                 break;
             // Refresh
             case R.id.ml_menu_refresh:
                 // TODO: factor this into each fragment
                 if(mCurrentFragment.equals("directories")) {
-                    DirectoryViewFragment directoryView = (DirectoryViewFragment) mSidebarAdapter.getFragment(mCurrentFragment);
+                    DirectoryViewFragment directoryView = (DirectoryViewFragment) getFragment(mCurrentFragment);
                     directoryView.refresh();
                 }
                 else if(mCurrentFragment.equals("history"))
-                    ((HistoryFragment)mSidebarAdapter.getFragment(mCurrentFragment)).refresh();
+                    ((HistoryFragment) getFragment(mCurrentFragment)).refresh();
                 else
-                    MediaLibrary.getInstance(this).loadMediaItems(this);
+                    MediaLibrary.getInstance(this).loadMediaItems(this, true);
                 break;
             // Open MRL
             case R.id.ml_menu_open_mrl:
@@ -432,6 +482,15 @@ public class MainActivity extends SherlockFragmentActivity {
         return super.onOptionsItemSelected(item);
     }
 
+    @Override
+    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
+        super.onActivityResult(requestCode, resultCode, data);
+        if (requestCode == ACTIVITY_RESULT_PREFERENCES) {
+            if (resultCode == PreferencesActivity.RESULT_RESCAN)
+                MediaLibrary.getInstance(this).loadMediaItems(this, true);
+        }
+    }
+
     private void reloadPreferences() {
         SharedPreferences sharedPrefs = getSharedPreferences("MainActivity", MODE_PRIVATE);
         mCurrentFragment = sharedPrefs.getString("fragment", "video");
@@ -475,9 +534,9 @@ public class MainActivity extends SherlockFragmentActivity {
             String action = intent.getAction();
 
             if (action.equalsIgnoreCase(ACTION_SHOW_PROGRESSBAR)) {
-                setProgressBarIndeterminateVisibility(Boolean.TRUE);
+                setSupportProgressBarIndeterminateVisibility(true);
             } else if (action.equalsIgnoreCase(ACTION_HIDE_PROGRESSBAR)) {
-                setProgressBarIndeterminateVisibility(Boolean.FALSE);
+                setSupportProgressBarIndeterminateVisibility(false);
             } else if (action.equalsIgnoreCase(ACTION_SHOW_TEXTINFO)) {
                 String info = intent.getStringExtra("info");
                 int max = intent.getIntExtra("max", 0);
@@ -506,11 +565,6 @@ public class MainActivity extends SherlockFragmentActivity {
         context.getApplicationContext().sendBroadcast(intent);
     }
 
-
-    public static void clearTextInfo(Context context) {
-        sendTextInfo(context, null, 0, 100);
-    }
-
     public static void sendTextInfo(Context context, String info, int progress, int max) {
         if (context == null)
             return;
@@ -522,6 +576,10 @@ public class MainActivity extends SherlockFragmentActivity {
         context.getApplicationContext().sendBroadcast(intent);
     }
 
+    public static void clearTextInfo(Context context) {
+        sendTextInfo(context, null, 0, 100);
+    }
+
     private void onOpenMRL() {
         AlertDialog.Builder b = new AlertDialog.Builder(this);
         final EditText input = new EditText(this);
