@@ -47,6 +47,7 @@ public class PreferencesActivity extends PreferenceActivity {
 
     public final static String NAME = "VlcSharedPreferences";
     public final static String LAST_MEDIA = "LastMedia";
+    public final static int RESULT_RESCAN = RESULT_FIRST_USER + 1;
 
     @SuppressWarnings("deprecation")
     @Override
@@ -54,6 +55,17 @@ public class PreferencesActivity extends PreferenceActivity {
         super.onCreate(savedInstanceState);
         addPreferencesFromResource(R.xml.preferences);
 
+        // Create onPrefChange
+        Preference rootDirectoryPref = findPreference("directories_root");
+        rootDirectoryPref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
+
+                    @Override
+                    public boolean onPreferenceChange(Preference preference, Object newValue) {
+                        setResult(RESULT_RESCAN);
+                        return true;
+                    }
+                });
+
         // Create onClickListen
         Preference directoriesPref = findPreference("directories");
         directoriesPref.setOnPreferenceClickListener(
@@ -63,6 +75,7 @@ public class PreferencesActivity extends PreferenceActivity {
                     public boolean onPreferenceClick(Preference preference) {
                         Intent intent = new Intent(getApplicationContext(), BrowserActivity.class);
                         startActivity(intent);
+                        setResult(RESULT_RESCAN);
                         return true;
                     }
                 });
@@ -99,7 +112,7 @@ public class PreferencesActivity extends PreferenceActivity {
                     @Override
                     public boolean onPreferenceClick(Preference preference) {
                         CheckBoxPreference checkboxHW = (CheckBoxPreference) preference;
-                        LibVLC.useIOMX(checkboxHW.isChecked());
+                        LibVLC.setIOMX(checkboxHW.isChecked());
                         return true;
                     }
                 });
@@ -138,12 +151,13 @@ public class PreferencesActivity extends PreferenceActivity {
         // Audio output
         ListPreference aoutPref = (ListPreference) findPreference("aout");
         int aoutEntriesId = Util.isGingerbreadOrLater() ? R.array.aouts : R.array.aouts_froyo;
+        int aoutEntriesIdValues = Util.isGingerbreadOrLater() ? R.array.aouts_values : R.array.aouts_values_froyo;
         aoutPref.setEntries(aoutEntriesId);
-        aoutPref.setEntryValues(aoutEntriesId);
+        aoutPref.setEntryValues(aoutEntriesIdValues);
         aoutPref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
             @Override
             public boolean onPreferenceChange(Preference preference, Object newValue) {
-                LibVLC.setAout(PreferencesActivity.this, (String) newValue, true);
+                LibVLC.setAout(PreferencesActivity.this, Integer.valueOf((String) newValue), true);
                 return true;
             }
         });
@@ -183,13 +197,13 @@ public class PreferencesActivity extends PreferenceActivity {
 
     @Override
     protected void onResume() {
-        AudioServiceController.getInstance().bindAudioService(this);
         super.onResume();
+        AudioServiceController.getInstance().bindAudioService(this);
     }
 
     @Override
     protected void onPause() {
-        AudioServiceController.getInstance().unbindAudioService(this);
         super.onPause();
+        AudioServiceController.getInstance().unbindAudioService(this);
     }
 }
