@@ -90,6 +90,9 @@ public class AudioBrowserFragment extends SherlockFragment implements ISortable
     public final static int MODE_SONG = 2;
     public final static int MODE_GENRE = 3;
 
+    /* All subclasses of Fragment must include a public empty constructor. */
+    public AudioBrowserFragment() { }
+
     @Override
     public void onCreate(Bundle savedInstanceState) {
         super.onCreate(savedInstanceState);
@@ -329,11 +332,11 @@ public class AudioBrowserFragment extends SherlockFragment implements ISortable
 
     @Override
     public void onDestroy() {
+        super.onDestroy();
         mSongsAdapter.clear();
         mArtistsAdapter.clear();
         mAlbumsAdapter.clear();
         mGenresAdapter.clear();
-        super.onDestroy();
     }
 
     private final ViewSwitchListener mViewSwitchListener = new ViewSwitchListener() {
