@@ -101,16 +101,16 @@ public class AudioPlayerActivity extends Activity implements IAudioPlayer {
 
     @Override
     protected void onResume() {
+        super.onResume();
         AudioServiceController.getInstance().bindAudioService(this);
         mAudioController.addAudioPlayer(this);
-        super.onResume();
     }
 
     @Override
     protected void onPause() {
+        super.onPause();
         mAudioController.removeAudioPlayer(this);
         AudioServiceController.getInstance().unbindAudioService(this);
-        super.onPause();
     }
 
     public static void start(Context context) {
@@ -118,6 +118,10 @@ public class AudioPlayerActivity extends Activity implements IAudioPlayer {
     }
 
     public static void start(Context context, Boolean dontParse) {
+        if (context == null) {
+            Log.e(TAG, "No context when starting AudioPlayerActivity");
+            return;
+        }
         Intent intent = new Intent(context, AudioPlayerActivity.class);
         intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
         if (dontParse)
@@ -249,6 +253,9 @@ public class AudioPlayerActivity extends Activity implements IAudioPlayer {
 
     public void onStopClick(View view) {
     	mAudioController.stop();
+    	Intent intent = new Intent(this, MainActivity.class);
+        intent.addFlags(Intent.FLAG_ACTIVITY_REORDER_TO_FRONT);
+        startActivity(intent);
     	finish();
     }
 
