@@ -35,6 +35,7 @@ import android.view.LayoutInflater;
 import android.view.View;
 import android.view.ViewGroup;
 import android.widget.ArrayAdapter;
+import android.widget.GridView;
 import android.widget.ImageView;
 import android.widget.TextView;
 
@@ -46,6 +47,7 @@ public class VideoListAdapter extends ArrayAdapter<Media>
     private int mSortDirection = 1;
     private int mSortBy = SORT_BY_TITLE;
     private String mLastMRL;
+    private boolean mListMode = false;
 
     public VideoListAdapter(Context context) {
         super(context, 0);
@@ -122,27 +124,32 @@ public class VideoListAdapter extends ArrayAdapter<Media>
      */
     @Override
     public View getView(int position, View convertView, ViewGroup parent) {
-
         ViewHolder holder;
         View v = convertView;
-        if (v == null) {
+        if (v == null || (((ViewHolder)v.getTag()).listmode != mListMode)) {
             LayoutInflater inflater = (LayoutInflater) getContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
-            v = inflater.inflate(R.layout.video_list_item, parent, false);
+            if (!mListMode)
+                v = inflater.inflate(R.layout.video_grid_item, parent, false);
+            else
+                v = inflater.inflate(R.layout.video_list_item, parent, false);
             holder = new ViewHolder();
             holder.layout = v.findViewById(R.id.layout_item);
             holder.thumbnail = (ImageView) v.findViewById(R.id.ml_item_thumbnail);
             holder.title = (TextView) v.findViewById(R.id.ml_item_title);
             holder.subtitle = (TextView) v.findViewById(R.id.ml_item_subtitle);
+            holder.listmode = mListMode;
             v.setTag(holder);
+            /* Set the layoutParams based on the values set in the video_grid_item.xml root element */
+            v.setLayoutParams(new GridView.LayoutParams(v.getLayoutParams().width, v.getLayoutParams().height));
         } else
             holder = (ViewHolder) v.getTag();
 
         Media media = getItem(position);
-        Util.setItemBackground(holder.layout, position);
         holder.title.setText(media.getTitle());
 
         Bitmap thumbnail;
         if (media.getPicture() != null) {
+            //FIXME Warning: the thumbnails are upscaled in the grid view!
             thumbnail = media.getPicture();
             holder.thumbnail.setImageBitmap(thumbnail);
         } else {
@@ -167,7 +174,7 @@ public class VideoListAdapter extends ArrayAdapter<Media>
             text = String.format("%s",
                     Util.millisToString(media.getLength()));
         }
-        
+
         if (media.getWidth() > 0 && media.getHeight() > 0) {
             text += String.format(" - %dx%d", media.getWidth(), media.getHeight());
         }
@@ -178,10 +185,15 @@ public class VideoListAdapter extends ArrayAdapter<Media>
     }
 
     static class ViewHolder {
+        boolean listmode;
         View layout;
         ImageView thumbnail;
         TextView title;
         TextView subtitle;
         ImageView more;
     }
+
+    public void setListMode(boolean value) {
+        mListMode = value;
+    }
 }
