@@ -212,8 +212,8 @@ public class VideoPlayerActivity extends Activity {
             public void onScrollStarted() {
                 showOverlay(OVERLAY_INFINITE);
                 /*FIXME
-                 * The setTracksAndSubtitles function probably doesn't work in case of many many Tracks and Subtitles
-                 * Moreover, in a video flux, if Tracks & Subtitles change, they won't be updated
+                 * The setTracksAndSubtitles method probably doesn't work in case of many many Tracks and Subtitles
+                 * Moreover, in a video stream, if Tracks & Subtitles change, they won't be updated
                  */
                 setTracksAndSubtitles();
             }
@@ -316,10 +316,11 @@ public class VideoPlayerActivity extends Activity {
 
     @Override
     protected void onPause() {
-        if(mSwitchingView) {
-            super.onPause();
+        super.onPause();
+
+        if(mSwitchingView)
             return;
-        }
+
         long time = mLibVLC.getTime();
         long length = mLibVLC.getLength();
         //remove saved position if in the last 5 seconds
@@ -358,11 +359,11 @@ public class VideoPlayerActivity extends Activity {
                     DatabaseManager.mediaColumn.MEDIA_TIME,
                     time);
         }
-        super.onPause();
     }
 
     @Override
     protected void onDestroy() {
+        super.onDestroy();
         unregisterReceiver(mBatteryReceiver);
         if (mLibVLC != null && !mSwitchingView) {
             mLibVLC.stop();
@@ -379,11 +380,11 @@ public class VideoPlayerActivity extends Activity {
             AudioPlayerActivity.start(this, true);
         }
         //AudioServiceController.getInstance().unbindAudioService(this);
-        super.onDestroy();
     }
 
     @Override
     protected void onResume() {
+        super.onResume();
         AudioServiceController.getInstance().bindAudioService(this);
 
         load();
@@ -404,8 +405,15 @@ public class VideoPlayerActivity extends Activity {
             }}, 500);
 
         showOverlay();
+    }
 
-        super.onResume();
+    @Override
+    public void onBackPressed() {
+        if (mOverlaySlider.isOpened()){
+            mOverlaySlider.animateClose();
+            return;
+        }
+        super.onBackPressed();
     }
 
     public static void start(Context context, String location) {
@@ -594,7 +602,7 @@ public class VideoPlayerActivity extends Activity {
                     activity.handleVout(msg);
                     break;
                 default:
-                    Log.e(TAG, "Event not handled");
+                    Log.e(TAG, String.format("Event not handled (0x%x)", msg.getData().getInt("event")));
                     break;
             }
             activity.updateOverlayPausePlay();
@@ -854,7 +862,7 @@ public class VideoPlayerActivity extends Activity {
         getWindow().setAttributes(lp);
         mIsFirstBrightnessGesture = false;
     }
-    
+
     private void doBrightnessTouch(float gesturesize) {
         // No Brightness action if gesturesize < 0.4 cm
         if (Math.abs(gesturesize) < 0.4)
@@ -1094,6 +1102,7 @@ public class VideoPlayerActivity extends Activity {
         updateOverlayPausePlay();
     }
 
+
     /**
      * hider overlay
      */
